﻿namespace Brizco.Repository.Handlers.Activity;

public class CreateActivityCommandHandler : IRequestHandler<CreateActivityCommand, ActivityLDto>
{
    private readonly IRepositoryWrapper _repositoryWrapper;
    private readonly ICurrentUserService _currentUserService;

    public CreateActivityCommandHandler(IRepositoryWrapper repositoryWrapper,ICurrentUserService currentUserService)
    {
        _repositoryWrapper = repositoryWrapper;
        _currentUserService = currentUserService;
    }

    public async Task<ActivityLDto> Handle(CreateActivityCommand request, CancellationToken cancellationToken)
    {
        if (_currentUserService.ComplexId == null)
            throw new AppException("ComplexId is null", ApiResultStatusCode.NotFound);
        if (!Guid.TryParse(_currentUserService.ComplexId, out Guid complexId))
            throw new AppException("ComplexId is wrong", ApiResultStatusCode.NotFound);
        try
        {
            await _repositoryWrapper.BeginTransaction(cancellationToken);
            var task = Domain.Entities.Task.Activity
                .Create(
                    request.Status,
                    request.DoneAt,
                    request.PerformanceDescription,
                    request.Title,
                    request.Description,
                    request.Type,
                    request.IsRelatedToShift,
                    request.IsRelatedToRole,
                    request.IsRelatedToPerson,
                    request.IsDisposable,
                    request.SetFor,
                    request.HasDisposed,
                    request.Amount,
                    request.AmountType,
                    complexId);

            if (task.IsRelatedToPerson)
            {
                if (request.Users.Count == 0)
                    throw new AppException("اگر فعالیت برای یک فرد انتخاب شده باشد باید لیست افراد را ارسال نمایید");
                task.AddUser(request.Users.ToArray());
            }

            if (task.IsRelatedToRole)
            {
                if (request.Roles.Count == 0)
                    throw new AppException(
                        "اگر فعالیت برای یک گروه نقش انتخاب شده باشد باید لیست نقش ها را ارسال نمایید");
                task.AddShift(request.Roles.ToArray());
            }

            if (task.IsRelatedToShift)
            {
                if (request.Shifts.Count == 0)
                    throw new AppException("اگر فعالیت برای یک شیفت انتخاب شده باشد باید لیست شیفت ها را ارسال نمایید");
                task.AddRole(request.Shifts.ToArray());
            }

            _repositoryWrapper.SetRepository<Domain.Entities.Task.Activity>().Add(task);
            await _repositoryWrapper.SaveChangesAsync(cancellationToken);
            await _repositoryWrapper.CommitAsync(cancellationToken);
            return task.AdaptToLDto();
        }
        catch (Exception)
        {
            await _repositoryWrapper.RollBackAsync(cancellationToken);
            throw;
        }
    }
}