﻿using Brizco.Domain.Entities.User;
using Microsoft.AspNetCore.Identity;

namespace Brizco.Repository.Handlers.Complex;

public class CreateComplexUserCommandHandler : IRequestHandler<CreateComplexUserCommand, ComplexUserSDto>
{
    private readonly IRepositoryWrapper _repositoryWrapper;
    private readonly UserManager<ApplicationUser> _userManager;
    private readonly RoleManager<ApplicationRole> _roleManager;
    private readonly ILogger<CreateComplexUserCommandHandler> _logger;

    public CreateComplexUserCommandHandler(IRepositoryWrapper repositoryWrapper,
        UserManager<ApplicationUser> userManager,
        RoleManager<ApplicationRole> roleManager,
        ILogger<CreateComplexUserCommandHandler> logger)
    {
        _repositoryWrapper = repositoryWrapper;
        _userManager = userManager;
        _roleManager = roleManager;
        _logger = logger;
    }

    public async Task<ComplexUserSDto> Handle(CreateComplexUserCommand request, CancellationToken cancellationToken)
    {
        try
        {
            await _repositoryWrapper.BeginTransaction(cancellationToken);
            var complex = await _repositoryWrapper.SetRepository<Domain.Entities.Complex.Complex>().TableNoTracking
                .FirstOrDefaultAsync(c => c.Id == request.ComplexId, cancellationToken);
            if (complex == null)
                throw new AppException("Complex not found", ApiResultStatusCode.NotFound);

            var complexUser = complex.AddComplexUser(request.UserId, request.RoleId);
            _repositoryWrapper.SetRepository<Domain.Entities.Complex.Complex>().Update(complex);
            var role = await _roleManager.FindByIdAsync(request.RoleId.ToString());
            var user = await _userManager.FindByIdAsync(request.UserId.ToString());
            var result = await _userManager.AddToRoleAsync(user, role.Name);
            if (!result.Succeeded)
                throw new AppException(string.Join('|', result.Errors));
            await _repositoryWrapper.SaveChangesAsync(cancellationToken);
            await _repositoryWrapper.CommitAsync(cancellationToken);
            return complexUser.AdaptToSDto();
        }
        catch (Exception)
        {
            await _repositoryWrapper.RollBackAsync(cancellationToken);
            throw;
        }
    }
}