﻿namespace Brizco.Repository.Handlers.Shift;

public class UpdateShiftCommandHandler : IRequestHandler<UpdateShiftCommand, bool>
{
    private readonly IRepositoryWrapper _repositoryWrapper;
    private readonly ICurrentUserService _currentUserService;

    public UpdateShiftCommandHandler(IRepositoryWrapper repositoryWrapper,ICurrentUserService currentUserService)
    {
        _repositoryWrapper = repositoryWrapper;
        _currentUserService = currentUserService;
    }

    public async Task<bool> Handle(UpdateShiftCommand request, CancellationToken cancellationToken)
    {
        var shift = await _repositoryWrapper.SetRepository<Domain.Entities.Shift.Shift>()
            .TableNoTracking.FirstOrDefaultAsync(s => s.Id == request.Id);
        if (shift == null)
            throw new AppException("Shift not found", ApiResultStatusCode.NotFound);

        if (_currentUserService.ComplexId == null)
            throw new AppException("ComplexId is null", ApiResultStatusCode.NotFound);
        if (!Guid.TryParse(_currentUserService.ComplexId, out Guid complexId))
            throw new AppException("ComplexId is wrong", ApiResultStatusCode.NotFound);

        var newShift = Domain.Entities.Shift.Shift.Create(request.Title,
            request.Description,
            request.StartAt,
            request.EndAt,
            complexId);
        newShift.Id = request.Id;
        _repositoryWrapper.SetRepository<Domain.Entities.Shift.Shift>()
            .Update(newShift);
        await _repositoryWrapper.SaveChangesAsync(cancellationToken);

        return true;
    }
}