﻿namespace Brizco.Repository.Handlers.Task;

public class UpdateActivityCommandHandler : IRequestHandler<UpdateTaskCommand, bool>
{
    private readonly IRepositoryWrapper _repositoryWrapper;
    private readonly ICurrentUserService _currentUserService;

    public UpdateActivityCommandHandler(IRepositoryWrapper repositoryWrapper,ICurrentUserService currentUserService)
    {
        _repositoryWrapper = repositoryWrapper;
        _currentUserService = currentUserService;
    }

    public async Task<bool> Handle(UpdateTaskCommand request, CancellationToken cancellationToken)
    {
        var task = await _repositoryWrapper.SetRepository<Domain.Entities.Task.Task>()
            .TableNoTracking.FirstOrDefaultAsync(s => s.Id == request.Id,cancellationToken);
        if (task == null)
            throw new AppException("Task not found", ApiResultStatusCode.NotFound);

        if (_currentUserService.ComplexId == null)
            throw new AppException("ComplexId is null", ApiResultStatusCode.NotFound);
        if (!Guid.TryParse(_currentUserService.ComplexId, out Guid complexId))
            throw new AppException("ComplexId is wrong", ApiResultStatusCode.NotFound);

        var newTask = Domain.Entities.Task.Task.Create(request.Title,
            request.Description,
            request.Type,
            request.IsRelatedToShift,
            request.IsRelatedToRole,
            request.IsRelatedToPerson,
            request.IsDisposable,
            request.SetFor,
            request.HasDisposed,
            request.Amount,
            request.AmountType,
            complexId);
        newTask.Id = request.Id;

        _repositoryWrapper.SetRepository<Domain.Entities.Task.Task>()
            .Update(newTask);
        await _repositoryWrapper.SaveChangesAsync(cancellationToken);

        return true;
    }
}